package core

import (
	"git.akyoto.dev/cli/q/src/build/errors"
)

// Compile waits for the scan to finish and compiles all functions.
func Compile(functions <-chan *Function, errs <-chan error) (Result, error) {
	result := Result{}
	allFunctions := map[string]*Function{}

	for functions != nil || errs != nil {
		select {
		case err, ok := <-errs:
			if !ok {
				errs = nil
				continue
			}

			return result, err

		case function, ok := <-functions:
			if !ok {
				functions = nil
				continue
			}

			function.functions = allFunctions
			allFunctions[function.Name] = function
		}
	}

	// Start parallel compilation
	CompileAllFunctions(allFunctions)

	// Report errors if any occurred
	for _, function := range allFunctions {
		if function.err != nil {
			return result, function.err
		}

		result.InstructionCount += len(function.assembler.Instructions)
	}

	// Check for existence of `main`
	main, exists := allFunctions["main"]

	if !exists {
		return result, errors.MissingMainFunction
	}

	result.Main = main
	result.Functions = allFunctions
	return result, nil
}
