package core

import (
	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/cpu"
	"git.akyoto.dev/cli/q/src/build/errors"
	"git.akyoto.dev/cli/q/src/build/token"
)

// ExecuteRegisterRegister performs an operation on two registers.
func (f *Function) ExecuteRegisterRegister(operation token.Token, destination cpu.Register, source cpu.Register) error {
	switch operation.Text() {
	case "+", "+=":
		f.assembler.RegisterRegister(asm.ADD, destination, source)

	case "-", "-=":
		f.assembler.RegisterRegister(asm.SUB, destination, source)

	case "*", "*=":
		f.assembler.RegisterRegister(asm.MUL, destination, source)

	case "/", "/=":
		f.assembler.RegisterRegister(asm.DIV, destination, source)

	case "=":
		if destination != source {
			f.assembler.RegisterRegister(asm.MOVE, destination, source)
		}

	default:
		return errors.New(&errors.InvalidOperator{Operator: operation.Text()}, f.File, operation.Position)
	}

	return nil
}
